/* ------------------------------------------------------------------------ */
/* --                Main module (Calendar Service)                      -- */
/* --                This script handles all CGI requests                -- */
/* --                                                                    -- */
/* --                Thomas Jungmann thj@gmx.net                         -- */
/* --                last modified 03.12.2000                            -- */
/* --                                                                    -- */
/* ------------------------------------------------------------------------ */

call dbgout "Calendar CGI started "                     --for debug purposes only
                                                        --writes debug info to a file
                                                        
call initVars                                           --inits all handles to the interface classes (DB, etc.)
                                                        --and puts the CGI-Variables into cgi.
                                                        
html~printHeader('The Web Calendar')                    --print MIME-Type and HTML Header

select
    when cgi.action = 'logon'       then call logon
    when cgi.action = 'newuser'     then call newuser
    when cgi.action = 'logout'      then call logout
    when cgi.action = 'overview'    then call overview
    when cgi.action = 'goto'        then call gotoMonth
    when cgi.action = 'gotoform'    then call gotoForm
    when cgi.action = 'viewday'     then call viewDay
    when cgi.action = 'view'        then call viewEvent
    when cgi.action = 'create'      then call createEvent
    when cgi.action = 'newentry'    then call newentryForm
    when cgi.action = 'Edit'        then call editEventForm
    when cgi.action = 'update'      then call updateEvent
    when cgi.action = 'Delete'      then call deleteEvent
    otherwise call abort 'Unknown CGI-Action'
end

html~printTail                                          --complete the HTML page

DROP db                                                 --drop references to the interface classes
DROP html
DROP parser
call dbgout "CGI-Script successfully completed"
exit 0

-----------END OF MAIN PROGRAM--------------

--------------------------------------------------------------
/*                          Subroutines                     */
--------------------------------------------------------------

---------------------------
/* Handles the logon request */
---------------------------
logon:  PROCEDURE EXPOSE html db cgi.                   --cgi. keeps all CGI-Variables, e.g. 
                                                        --?name=peter -> is stored in cgi.name
    call dbgout "logon()"
    uid = db~getUserID(cgi.name)                        --find User ID for given name
    select
        when uid = -1 then call abort 'DB-Access error while retrieving UserID'
        when uid = -2 then call abort 'Username not found'
        otherwise nop
    end
    dbpwd = db~getUserPwd(uid)                          --get Users Password
    select
        when dbpwd = -1 then call abort 'DB-Access error while retrieving UserPassword'
        when dbpwd = -2 then call abort 'UserID not found'
        otherwise nop
    end
    if \(dbpwd == cgi.pwd) then                         --Pwd. does not match to Pwd. stored in Database
        do
            html~out(html~paragraph('Passwords must be entered exactly, as they are case sensitive!'))
            call abort 'Password incorrect'
        end
    newSID = db~createSID(uid)                          --create a new session identifier
    if newSID = -1 then                                 --problem with the database?
        call abort 'Could not create SID'
    cgi.sid = newSID                                    --store SID in global variable, 
                                                        --for later user (for procedure overview)
    db~deleteAllExpiredSID()                            --delete old and expired SIDs - for cleanup only

    call overview                                       --overview shows the month-overview (=start page)
    return

---------------------------
/* Handles the logoff request */
---------------------------
logout:     PROCEDURE EXPOSE html db cgi.
    call dbgout "logout()"
    res = db~closeSession(cgi.sid)                      --delete entry for current session in the database
    if res \=0 then call abort 'SID could not be deleted'   --perhaps expired and then deleted in a cleanup process
    html~out(html~headline('You have successfully logged out, thank you for using THE WEB CALENDAR',3))
    return

---------------------------
/* Handles the input from 'newuser.html' */
---------------------------
newuser:    PROCEDURE EXPOSE html db cgi.
    call dbgout "newUser()"
    if cgi.name == "CGI.NAME" then call abort 'Please provide a valid username'     --cgi.name undefined, no name given at all
    if symbol(cgi.name) == 'BAD' then call abort 'Please do not use any special characters in your username'
    if cgi.pwd  == "CGI.PWD" then call abort 'Please provide a valid password'      --cgi.pwd undef., nothing was entered
    if symbol(cgi.pwd)  == 'BAD' then call abort 'Please do not use any special characters in your password'
    res = db~createNewUser(cgi.name,cgi.pwd)            --insert values into databases 'user'-relation

    if res < 0 then                                     --error? probably the username is already in the database,
        do                                              --but could also indicate some other DB-Error...
            html~out(html~paragraph('Username already exists. Please use another name.'))
            html~out(html~hyperlink('../newuser.html','Try again...'))
            call abort 'Username exists'
        end
    html~out(html~paragraph('You have successfully created a new user account, you may now log on to THE WEB CALENDAR'))
    return

---------------------------
/* This procedure displays the current month as overview */
---------------------------
overview:   PROCEDURE EXPOSE html db cgi.
    call dbgout "overview()"
    uid = db~getUID4SID(cgi.sid)                        --Find userID for given session ID
    if uid < 0 then call abort 'Session has expired, please log on again'
    html~printNav(cgi.sid)                              --Print HTML-Head with links for the main actions

    call viewMonth uid,date('S',,,'-')                  --show overview for given user and current date
    return

---------------------------
/* This procedure displays any month between 01.01.0001 and 31.12.9999
   according to the gegorian calendar */
---------------------------
viewMonth:  PROCEDURE EXPOSE html db cgi.
    call dbgout "viewMonth()"
    parse arg uid,dt
    calendar. = getCalendar(uid,dt)                     --this array consists of 4 to 5 textlines
                                                        --already formated as days of a specifiv month

    dtLang = date('L',dt,'S',,'-')                      --convert date to english language format

    parse var dtLang . month year                       --get month & year for the calendar headline
    prevDt = getPrevMonth(dt)                           --get previous and next month for the hyperlinks
    nextDt = getNextMonth(dt)                           --next to the headline...
    parse var prevDt pYear '-' pMonth '-' .             --... and split the date
    parse var nextDt nYear '-' nMonth '-' .
    
                                                        --prepare the hyperlinks for previous and next month
    pm = html~hyperlink('calserv.cgi?action=goto&year='||pYear||'&month='||pMonth||'&sid='||cgi.sid,'&lt')
    nm = html~hyperlink('calserv.cgi?action=goto&year='||nYear||'&month='||nMonth||'&sid='||cgi.sid,'&gt;')
    html~out('<table border="0" cellpadding="15"><tr>')
    html~out('<tr><td bgcolor= "#FFFF99" valign="TOP">')
    html~out(html~headline(pm||' 'month||' '||year||' '||nm,1,'CENTER'))    --headline output
    html~out('</td></tr>')
    html~out('<td bgcolor="#FFFFCC" valign="TOP" align="LEFT">')
    html~out(html~fixedSpaceStart)                      --ensure that a fixed space font is used, so
                                                        --things don't get messed up
    html~out('<font size="4">')
    html~out('Mon Tue Wed Thu Fri Sat Sun')
    html~out('---------------------------')
    do i=1 to calendar.0                                --output of the formerly prepared calendar array
        html~out(calendar.i)
        if i <> calendar.0 then html~out('<br>')
    end
    html~out('</font>')
    html~out(html~fixedSpaceEnd)
    html~out('</td></tr></table>')
    call gotoForm                                       --display a small form for quick jump to other month/year
    return

---------------------------
/* Show the overview of all events on one day */
---------------------------
viewDay:    PROCEDURE EXPOSE html db cgi.
    call dbgout "viewDay()"
    uid = db~getUID4SID(cgi.sid)                        --Find userID for given session ID
    if uid < 0 then call abort 'Session has expired, please log on again'
    html~printNav(cgi.sid)                              --Print HTML-Head with links for the main actions
    events. = db~getEventsOfDay(uid,cgi.date)           --events. -array has now all events for the day
    if events.eventID.0 = 0 then html~out(html~paragraph(date('L',cgi.date,'S',,'-')||': There are no events on schedule for this day: '))
    else
        do
            html~out('<table border="0" cellpadding="5">')
                                                                            --headline with date
            html~out('<tr><td valign="TOP" align="CENTER" colspan="2">'||html~headline(date('L',cgi.date,'S',,'-'),2,'CENTER')||'</td>')
            
            do i=1 to events.eventID.0                                      --loop over all events
                html~out('<tr>')
                hl = 'calserv.cgi?action=view&eventid='||events.eventID.i||'&sid='||cgi.sid  --prepare hyperlink for event
                html~out(html~tabdata(events.eventTime.i,i//2))             --i mod 2 = alternating bgcolor
                html~out(html~tabdata(html~hyperlink(hl,events.description.i),i//2))
                html~out('</tr>')
            end
            html~out('</table>')
        end
    return

---------------------------
/* Show an event with all details */
---------------------------
viewEvent:      PROCEDURE EXPOSE html db cgi.
    call dbgout "viewEvent()"
    sq = d2c(39)
    checkUID = db~getUID4SID(cgi.sid)                   --Find userID for given session ID
    if checkUID < 0 then call abort 'Session has expired, please log on again'
    html~printNav(cgi.sid)                              --Print HTML-Head with links for the main actions
    event. = db~retrieveEvent(cgi.eventid)              --get event from the database and store it in event. array
    if event.eventID.0 = 0 then call abort 'Event has not been found'
    if event.uID.1 \= checkUID then call abort 'Event does not belong to user'      --if someone has tampered the
                                                                    --query string to get information from other users
    
    html~out('<table border="0" cellpadding="5">')
    html~out('<tr>')
    html~out(html~tabdata('Date',0))
    html~out(html~tabdata(event.eventDate.1,0))
    html~out('</tr>')
    html~out('<tr>')
    html~out(html~tabdata('Time',1))
    html~out(html~tabdata(event.eventTime.1,1))
    html~out('</tr>')
    html~out('<tr>')
    html~out(html~tabdata('Description',0))
    html~out(html~tabdata(event.description.1,0))
    html~out('</tr>')
    html~out('<tr>')
    html~out(html~tabdata('Category',1))
    html~out(html~tabdata(db~getCategory(event.categoryID.1),1))
    html~out('</tr>')
    html~out('<tr>')
    html~out(html~tabdata('Duration',0))
    html~out(html~tabdata(event.duration.1,0))
    html~out('</tr>')
    html~out('</table>')
    
                                                    --this form displays the 'Edit' and 'Delete' buttons
    html~out('<FORM ACTION='||sq||'calserv.cgi'||sq||' METHOD="GET">')
    html~out('<INPUT TYPE='||sq||'HIDDEN'||sq||' NAME='||sq||'eventid'||sq||' VALUE='||cgi.eventid||'>')
    html~out('<INPUT TYPE='||sq||'HIDDEN'||sq||' NAME='||sq||'sid'||sq||' VALUE='||cgi.sid||'>')
    html~out('<INPUT TYPE='||sq||'SUBMIT'||sq||' NAME='||sq||'action'||sq||' VALUE='||sq||'Edit'||sq||'>')
    html~out('<INPUT TYPE='||sq||'SUBMIT'||sq||' NAME='||sq||'action'||sq||' VALUE='||sq||'Delete'||sq||'>')
    html~out('</FORM>')
    return

---------------------------
/* This procedures creates a HTML-Form for new entries or for 
   existing entries that are to be modified */
---------------------------
newEntryForm:    PROCEDURE EXPOSE html db cgi.
                                                    --in case of a modification of an existing
                                                    --event, these default arguments get their values
                                                    --from the procedure 'editEventForm'
    use arg defEventID,defDay,defMonth,defYear,defHour,defMinute,defDHour,defDMinute,defDescription,defCategory
    call dbgout "newEntry()"
    sq = d2c(39)                                    -- decimal to char, 39 = single quote
    
    uid = db~getUID4SID(cgi.sid)                    --Find userID for given session ID
    if uid < 0 then call abort 'Session has expired, please log on again'
    html~printNav(cgi.sid)                          --Print HTML-Head with links for the main actions
    
    
    if defDay=='DEFDAY' then                        --not default values given, creating some
        do
            html~out(html~headline('Create a new entry',3))
            dt = date('S')                                  --current date and time as the default
            parse var dt defYear 5 defMonth 7 defDay
            tm = time()
            parse var tm defHour ':' defMinute ':' .
            defDHour = 0
            defDMinute = 0
            defDescription = ''
            defCategory = 1
            action = 'create'                       --prepare 'hidden field' 'action'
        end
    else
        do
            html~out(html~headline('Edit event',3))
            action = 'update'                       --prepare 'hidden field' 'action'
        end

    html~out('<FORM ACTION='||sq||'calserv.cgi'||sq||' METHOD="GET">')      --start form
    html~out('<INPUT TYPE='||sq||'HIDDEN'||sq||' NAME='||sq||'action'||sq||' VALUE='||sq||action||sq||'>')
    if action='update' then                         --for update an eventID must be supplied
        html~out('<INPUT TYPE='||sq||'HIDDEN'||sq||' NAME='||sq||'eventid'||sq||' VALUE='||sq||defEventID||sq||'>')
    html~out('Date: <SELECT NAME='||sq||'day'||sq||' SIZE=1>')  --single line select box (dropdown) for 'day'-value
    do i=1 to 31
        padi = right(i,2,'0')                               --ensures that numbers < 10 are expanded (padded)
                                                            --with a preceding '0', so they are alway 2 chars long
        if i=defDay then
            html~out('<OPTION SELECTED VALUE='||sq||padi||sq||'>'||padi)        --this is the default
        else
            html~out('<OPTION VALUE='||sq||padi||sq||'>'||padi)             --these are all others
    end
    html~out('</SELECT>&nbsp;')

    html~out('<SELECT NAME='||sq||'month'||sq||' SIZE=1>')      --dropdown-field for month
    do i=1 to 12
        padi = right(i,2,'0')                               --i padded with preceding '0'

        dt = date('L','00/'||padi||'/01','O')               --english names of the months
        parse var dt . month .                              --is stored in 'month'
        if i=defMonth then
            html~out('<OPTION SELECTED VALUE='||sq||padi||sq||'>'||month)           --default
        else
            html~out('<OPTION VALUE='||sq||padi||sq||'>'||month)                    --other
    end
    html~out('</SELECT>')
    
                                                            --textfield for the year
    html~out('&nbsp;<INPUT TYPE='||sq||'TEXT'||sq||' NAME='||sq||'year'||sq||' SIZE=4 VALUE='||sq||defYear||sq||'>')
    
    html~out('<P>Time: <SELECT NAME='||sq||'hour'||sq||' SIZE=1>')  --dropdown for time/hour
    do i=0 to 24
        padi = right(i,2,'0')                                   --i padded with preceding '0'
        if i=defHour then
            html~out('<OPTION SELECTED VALUE='||sq||padi||sq||'>'||padi)
        else
            html~out('<OPTION VALUE='||sq||padi||sq||'>'||padi)
    end
    html~out('</SELECT>&nbsp;')
    
    html~out('<SELECT NAME='||sq||'minute'||sq||' SIZE=1>')     --dropdown 'minute'
    do i=0 to 60
        padi = right(i,2,'0')
        if i=defMinute then
            html~out('<OPTION SELECTED VALUE='||sq||padi||sq||'>'||padi)
        else
            html~out('<OPTION VALUE='||sq||padi||sq||'>'||padi)
    end
    html~out('</SELECT>')
    
    html~out('&nbsp;Duration: <SELECT NAME='||sq||'dhour'||sq||' SIZE=1>')  --dropdown 'duration'/hour
    do i=0 to 24
        padi = right(i,2,'0')
        if i=defDHour then
            html~out('<OPTION SELECTED VALUE='||sq||padi||sq||'>'||padi)
        else
            html~out('<OPTION VALUE='||sq||padi||sq||'>'||padi)
    end
    html~out('</SELECT>&nbsp;')
    
    html~out('<SELECT NAME='||sq||'dminute'||sq||' SIZE=1>')                    --dropdown 'duration'/minutes
    do i=0 to 60
        padi = right(i,2,'0')
        if i=defDMinute then
            html~out('<OPTION SELECTED VALUE='||sq||padi||sq||'>'||padi)
        else
            html~out('<OPTION VALUE='||sq||padi||sq||'>'||padi)
    end
    html~out('</SELECT>')
    
    html~out('<P>Description:')                                         --textfield for event description
    html~out('<TEXTAREA ROWS=4 COLS=25 NAME='||sq||'description'||sq||'>'||defDescription||'</TEXTAREA>')
    
    maxCat = db~getMaxCategory()                                --number of existing categorys in database
    html~out('<P>Category: <SELECT NAME='||sq||'category'||sq||' SIZE=1>')  --dropdown for event category
    do i=1 to maxCat
        if i=defCategory then
            html~out('<OPTION SELECTED VALUE='||sq||i||sq||'>'||db~getCategory(i))
        else
            html~out('<OPTION VALUE='||sq||i||sq||'>'||db~getCategory(i))
    end
    html~out('</SELECT>')
                                                                        --Save/Reset buttons
    html~out('<P><INPUT TYPE='||sq||'SUBMIT'||sq||' VALUE='||sq||'Save'||sq||'>')
    html~out('<INPUT TYPE='||sq||'RESET'||sq||' VALUE='||sq||'Reset'||sq||'>')
    html~out('<INPUT TYPE='||sq||'HIDDEN'||sq||' NAME='||sq||'sid'||sq||' VALUE='||cgi.sid||'>')
    html~out('</FORM>')
    
    html~out('<FORM ACTION='||sq||'calserv.cgi'||sq||' METHOD="GET">')      --Cancel button (back to overview)
    html~out('<INPUT TYPE='||sq||'HIDDEN'||sq||' NAME='||sq||'action'||sq||' VALUE='||sq||'overview'||sq||'>')
    html~out('<INPUT TYPE='||sq||'HIDDEN'||sq||' NAME='||sq||'sid'||sq||' VALUE='||cgi.sid||'>')
    html~out('<INPUT TYPE='||sq||'SUBMIT'||sq||' VALUE='||sq||'Cancel'||sq||'>')
    html~out('</FORM>')
    return

---------------------------
/* This procedure actually creates an event that has been
   entered in the 'createEventForm' */
---------------------------
createEvent:    PROCEDURE EXPOSE html db cgi.
    call dbgout "createEvent()"
    sq = d2c(39)                            -- decimal to char, 39 = single quote
    uid = db~getUID4SID(cgi.sid)
    if uid < 0 then call abort 'Session has expired, please log on again'
    html~printNav(cgi.sid)

    eventDate = cgi.year||'-'||cgi.month||'-'||cgi.day          --put date together...
    eventTime = cgi.hour||':'||cgi.minute||':00'                --... and time. seconds are alway 00
    eventDuration = cgi.dhour||':'||cgi.dminute||':00'
    res = db~createEvent(uid,cgi.category,cgi.description,eventDate,eventTime,eventDuration)  --store data in DB
    if res \= 0 then call abort "Error while creating event"

    html~out(html~headline('Event successfully stored',3))
    html~continueButton(cgi.sid)                                --display 'continue'-button (to main overview)
    return

---------------------------
/* This is executed when a user clicks the 'delete'-button in the event view */
---------------------------
deleteEvent:    PROCEDURE EXPOSE html db cgi.
    call dbgout "deleteEvent()"
    checkUID = db~getUID4SID(cgi.sid)
    event. = db~retrieveEvent(cgi.eventid)              --event is retrieved from the database because
                                                        --we have to check if the eventID actually belongs
                                                        --to the user (we need only the user ID)
    if checkUID < 0 then call abort 'Session has expired, please log on again'
    if event.eventID.0 = 0 then call abort 'Event has not been found'
    if event.uID.1 \= checkUID then call abort 'Event does not belong to user'
    
    html~printNav(cgi.sid)
    res = db~deleteEvent(cgi.eventid)
    if res \= 0 then call abort "Error while creating event"
    html~out(html~headline('Event deleted'))
    html~continueButton(cgi.sid)                                --display 'continue'-button (to main overview)
    return

---------------------------
/* Create a form to modify an existing event.
   This just creates the default values for the 'newEventForm' procedure */
---------------------------
editEventForm:      PROCEDURE EXPOSE html db cgi.
    call dbgout "editEvent()"
    uid = db~getUID4SID(cgi.sid)
    call dbgout "uid: "||uid
    if uid < 0 then call abort 'Session has expired, please log on again'

    event. = db~retrieveEvent(cgi.eventid)                      --get the old event data for the default values
    dt = event.eventDate.1
    tm = event.eventTime.1
    dr = event.duration.1
    parse var dt year '-' month '-' day                         --split the event date into single variables
    parse var tm hour ':' minute ':' .                          --...and the event time
    parse var dr dhour ':' dminute ':' .                        --...and the event duration as well

    call newEntryForm event.eventID.1,day,month,year,hour,minute,dhour,dminute,event.description.1,event.categoryID.1
    return

---------------------------
/* This procedure actually does the update-job
   It gets its data from the 'newEntryForm'-procedure */
---------------------------
updateEvent: PROCEDURE EXPOSE html db cgi.
    call dbgout "updateEvent()"
    sq = d2c(39)
    checkUID = db~getUID4SID(cgi.sid)
    event. = db~retrieveEvent(cgi.eventid)              --old data needed for check is a user
                                                        --is allowed to change the data (i.e. belongs to him)
    if checkUID < 0 then call abort 'Session has expired, please log on again'
    if event.eventID.0 = 0 then call abort 'Event has not been found'
    if event.uID.1 \= checkUID then call abort 'Event does not belong to user'
    html~printNav(cgi.sid)

    eventDate = cgi.year||'-'||cgi.month||'-'||cgi.day          --put date together
    eventTime = cgi.hour||':'||cgi.minute||':00'                --...and time; seconds alway 00
    eventDuration = cgi.dhour||':'||cgi.dminute||':00'
    res = db~updateEvent(cgi.eventid,cgi.category,cgi.description,eventDate,eventTime,eventDuration)
    if res \= 0 then call abort "Error while updating event"

    html~out(html~headline('Event updated'))
    html~continueButton(cgi.sid)                                --display 'continue'-button
    return

---------------------------
/* Shows the 'goto'-month-form (under the main calendar view) */
---------------------------
gotoForm:   PROCEDURE EXPOSE html cgi.
    call dbgout "gotoForm()"
    sq = d2c(39)                            -- decimal to char, 39 = single quote
    html~out(html~headline('Choose month and year to display',4))
    html~out('<FORM ACTION='||sq||'calserv.cgi'||sq||' METHOD="GET">')
    html~out('<INPUT TYPE='||sq||'HIDDEN'||sq||' NAME='||sq||'action'||sq||' VALUE='||sq||'goto'||sq||'>')
    html~out('<SELECT NAME='||sq||'month'||sq||' SIZE=1>')
    dt = date('L')                                              --current date in english language
    parse var dt . thisMonth .                                  --e.g. thisMonth='January'
    do i=1 to 12
        padi = right(i,2,'0')                                   --expand i<10 to 2 digits with preceding '0'
        dt = date('L','00/'||padi||'/01','O')
        parse var dt . month .                                  --all monthnames in english language
        if month=thisMonth then
            html~out('<OPTION SELECTED VALUE='||sq||padi||sq||'>'||month)       --current month is default
        else
            html~out('<OPTION VALUE='||sq||padi||sq||'>'||month)
    end
    html~out('</SELECT>')

    html~out('&nbsp;<INPUT TYPE='||sq||'TEXT'||sq||' NAME='||sq||'year'||sq||' SIZE=4 VALUE="2000">')   --textfield 'year'

    html~out('&nbsp;<INPUT TYPE='||sq||'SUBMIT'||sq||' VALUE='||sq||'Go'||sq||'>')          --'Go'-button
    html~out('<INPUT TYPE='||sq||'HIDDEN'||sq||' NAME='||sq||'sid'||sq||' VALUE='||cgi.sid||'>')
    html~out('</FORM>')
    return

---------------------------
/* Navigate to a date given in the 'gotoForm' */
---------------------------
gotoMonth:      PROCEDURE EXPOSE html db cgi.
    call dbgout "gotoMonth()"
    uid = db~getUID4SID(cgi.sid)
    if uid < 0 then call abort 'Session has expired, please log on again'
    html~printNav(cgi.sid)
    if cgi.year <= 0 | cgi.year > 9999 then                     --range check for year
        do
            if length(cgi.year) <> 2 then                       --year=00 probably a century year meant
            do
                html~out(html~headline(d2c(39)||cgi.year||d2c(39)||' is not a valid year!',2))
                return
            end
        end
--  cgi.year = trunc(cgi.year)              -- just in case someone enters year 2001.17
    
    if length(cgi.year) = 2 then            -- nifty heuristic to guess which century the user
        do                                  -- meant, when he entered only the last 2 digits of the year
            thisYear = date()
            parse var thisYear . . thisYear
            diff = thisYear//100 - cgi.year                 --find the years within 50 years in the past
            select                                          --and 49 years in the future
                when diff>0 & abs(diff)>50 then cgi.year=thisYear+100-diff
                when diff>=0 & abs(diff)<=50 then cgi.year=thisYear-diff
                when diff<=0 & abs(diff)>=50 then cgi.year=thisYear-(100-abs(diff))
                when diff<0 & abs(diff)<50 then cgi.year=thisYear+abs(diff)
                otherwise nop
            end
        end
    dt = right(cgi.year,4,'0')||'-'||cgi.month||'-01'

    call viewMonth uid,dt                       --show the month-overview for that date
    return
    
---------------------------
/* This returns an array with 4 to 6 textlines, already formated
   like a calendar table with hyperlinks on days where the user 
   has at least one event on schedule */
---------------------------
getCalendar:     PROCEDURE EXPOSE db html cgi.
    call dbgout "getCalendar()"
    parse arg uid,year '-' month '-' day
    year  = right(year,4,'0')               --expanded to 4 digits (precedings '0')
    month = right(month,2,'0')              --expanded to 2 digits (precedings '0')
    day   = right(month,2,'0')
    dt = year||'-'||month||'-01'        --put a date-string together, 'day' is the first day of a month
    
    n = dayOfWeek(dt)                   --determine the weekday of the first day of a month (needed for heading spaces)

    today = date('S',,,'-')             --current date (will be marked as 'bold' in the calendar)

    out.1 = ' '||copies('    ',n)       --begin with the heading spaces (no spaces when monday)
    
    do i=1 to (7-n)                                 --loop for the first calendar line
        padi = right(i,2,'0')                       --expand to 2 digits, if necessary
        testdate = year||'-'||month||'-'||padi      --put a date string together for test if it is today
        boldS = ''                                  --clear the bold-start variable (empty=no bold tag on this day)
        boldE = ''                                  --clear the bold-end variable
        if testdate == today then
            do
                boldS = '<b>'                       --if today, prepare for bold printing
                boldE = '</b>'
            end
        events. = db~getEventsOfDay(uid,testdate)       --any events on testdate?
        if events.eventID.0 = 0 then
            out.1 = out.1||boldS||padi||boldE||'  '     --print without hyperlink (put probably bold, if today)
        else do
                                                        --event for testdate exists. prepare the hyperlink
                hyperlink = html~hyperlink('calserv.cgi?action=viewday&date='||testdate||'&sid='||cgi.sid,boldS||padi||boldE)
                out.1 = out.1||hyperlink||'  '
        end
    end
    
    line = 2                                        --initial value for other calendar lines
    s = 7-n+1                                       --first day on second line
    maxDays = daysInMonth(dt)                       --get number of days in a given month
    do until e = maxDays                            --e = end of calendar line (until expression is checked at end of loop)
        out.line = ' '                              --one heading space on each line
        e = s+6                                     --max. 7 days on each line
        if e > maxDays then e = maxDays             --...but limited to the maximum of days on this month

        do i=s to e                                 --loop over one week
            padi = right(i,2,'0')
            testdate = year||'-'||month||'-'||padi      --the same comment as above applies to these lines....
            boldS = ''
            boldE = ''
            if testdate == today then
                do
                    boldS = '<b>'
                    boldE = '</b>'
                end
            events. = db~getEventsOfDay(uid,testdate)
            if events.eventID.0 = 0 then
                out.line=out.line||boldS||padi||boldE||'  '
            else do
                    hyperlink = html~hyperlink('calserv.cgi?action=viewday&date='||testdate||'&sid='||cgi.sid,boldS||padi||boldE)
                    out.line = out.line||hyperlink||'  '
            end

        end
        line = line+1                                   --next line
        s = e+1                                         --last day+1 is the start day for the next line
    end
    out.0 = line-1                          --Linecount of calendar page. Usually 5 lines, but sometimes
                                            --in 28-Days-Februarys that start with a Monday only 4 (e.g. Feb.1999)
    return out.

---------------------------
/* This is called at the very beginning of this script
   and initializes the handles for the interface objects
   and reads the CGI-Variables from the query string */
---------------------------
initVars:   PROCEDURE EXPOSE db html cgi.
    call dbgout "initVars()"
    db     = .sqlInterface~NEW
    html   = .HTMLgen~NEW
    parser = .cgiParse~NEW
    cgi.   = parser~parseQueryString()
    return

---------------------------
/* Called when an error condition occurs and prints
   the reason as the HTML-Output */
---------------------------
abort:  PROCEDURE EXPOSE html
    parse arg why
    call dbgout "Abnormal program termination: "||why
    html~out(html~headline(why,1))
    html~printTail
    exit 0

---------------------------
/* This procedure is only for debugging purposes and
   writes a file called 'debug.dat' to the disk */
---------------------------
dbgout: PROCEDURE
    use arg out
    file = .stream~new('debug.dat')
    file~open('both append')
    file~lineout(date()||' '||time()||' '||out)
    file~close()
    return


---------------------------
/* Returns the number of days a given month has */
---------------------------
daysInMonth:    PROCEDURE
    call dbgout "daysInMonth()"
    parse arg year '-' month '-' day
    year  = right(year,4,'0')
    month = right(month,2,'0')
    day   = '01'
    dt = year||month||day
    dt = date('B',dt,'S')               --date('B') returns the number of days since 01.01.0001
    do until (month \= xmonth)          --this loop increases this value until it is switched to the next month
        res = date('S',dt,'B','-')
        parse var res . '-' xmonth '-'
        dt = dt + 1
    end
    dt = dt-2
    dt = date('S',dt,'B','-')           --retransformation to a 'normal' date format
    parse var dt . '-' . '-' dt         --only the day is important
    return dt

---------------------------
/* Returns the weekday of a given date */
---------------------------
dayOfWeek:      PROCEDURE
    call dbgout "dayOfWeek()"
    use arg dt
    parse var dt year '-' month '-' day
    year  = right(year,4,'0')
    month = right(month,2,'0')
    day   = right(day,2,'0')
    dt = year||month||day
    dt = date('B',dt,'S')//7            --date('B') returns the number of days since 01.01.0001, this 
                                        --modulo 7 is the weekday
    return dt

---------------------------
/* Finds the previous month (for the navigation hyperlinks) */
---------------------------
getPrevMonth:      PROCEDURE
    call dbgout "getPrevMonth()"
    use arg dt
    parse var dt year '-' month '-' .
    year  = right(year,4,'0')
    month = right(month,2,'0')
    if month = '01' then                            --when month is January...
        do
            year = right(year-1,4,'0')
            month = '12'                            --...previous month is December, the year before
        end
    else
        month = right(month-1,2,'0')
    return (year||'-'||month||'-'||'01')

---------------------------
/* Finds the next month (for the navigation hyperlinks) */
---------------------------
getNextMonth:      PROCEDURE
    call dbgout "getNextMonth()"
    use arg dt
    parse var dt year '-' month '-' .
    year  = right(year,4,'0')
    month = right(month,2,'0')
    if month = '12' then                            --when month is December...
        do
            year = right(year+1,4,'0')
            month = '01'                            --next month is January, in the next year
        end
    else
        month = right(month+1,2,'0')
    return (year||'-'||month||'-'||'01')
    
--------------------------------------------------------------
/*                          Directives                      */
--------------------------------------------------------------
::REQUIRES 'cgilib.cmd'
::REQUIRES 'sqli.cmd'
::REQUIRES 'htmlgen.cmd'
