/*------------------------------------------------------
|   The "cgiParse"-class contains                       |
|   REXX Methods to handle CGI-Environment variables    |
|   e.g. parse the query-string, ...                    |
|   emasovic@wi-inf.uni-essen.de                        |
|   http://swt.wi-inf.uni-essen.de/~emasovic            |
|   based on CGIParse by Sacha Prins                    |
|   (slightly modified version) 24.11.2000              |
-------------------------------------------------------*/

::CLASS cgiParse PUBLIC                                     -- works only with CGI-GET-Method

    ::METHOD parseQueryString
        queryString = self~getEnv('QUERY_STRING')           --retrieve query string from the environment 
        queryString= TRANSLATE(queryString, ' ', '+')       --retranslate '+'-Symbols to Spaces
        DO WHILE LENGTH(queryString) > 0 
            varCouple= ''
            PARSE VAR queryString varCouple'&'queryString   --find parts between '&'-Symbol
            PARSE VAR varCouple varName'='varVal            --and split them into varName and varValue
            IF varName = '' | varVal= '' THEN ITERATE       --anything empty? => then find next part
            varName= 'cgi.' || self~urlDecode(varName)      --prepare stem-variable name, decode '%'-Expressions
            varVal=  self~urlDecode(varVal)                 --prepare value and decode '%'-Expressions
            IF SYMBOL(varName) = 'BAD' THEN ITERATE         --varName cannot be used as a Rexx-Variable, find next part

                                                            --multiple assignments to the same variable?
                                                            --retain old value and append new, separated by 'Linefeed'
            IF VALUE(varName) \= TRANSLATE(varName) THEN call VALUE varName, VALUE(varName) || '0a'x || varVal 
            ELSE call VALUE varName, varVal                 --assign the variable to the Rexx-Stem-Variable
        END
        RETURN cgi.                                         --return Stem Variable

    /*********************************************************************/ 
        :: METHOD URLDecode PRIVATE
            IF ARG()\=1 THEN RETURN ''                      --only one argument permitted
            line= ARG(1) 
            lineLen= LENGTH(line) 
            newLine= '' 
            i=1 
            DO WHILE i <= lineLen 
                c= SUBSTR(line, i, 1) 
                IF c \= '%' THEN newLine = newLine || c     --normal character? => then append
                ELSE IF i+2 <= lineLen THEN                 --is tested char = '%' (special char in query strings)?
                    DO 
                        newLine= newLine || x2c(SUBSTR(line, i+1, 2)) --append decoded character
                        i=i+2 
                    END 
                i= i+1 
            END 
            RETURN newLine                                  --return completely decoded string

    /*********************************************************************/ 
        ::METHOD getEnv PRIVATE
            RETURN VALUE(ARG(1),, 'ENVIRONMENT')            --retrieves variable from environment

    /*********************************************************************/ 
        ::METHOD putEnv PRIVATE                             --sets an environment variable
            RETURN VALUE(ARG(1), ARG(2), 'ENVIRONMENT')     --(not used here)
/*********************************************************************/